--[[
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * u should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 --]]

--  Ported to Lua/Love2d by unixfreak (ricky thomson)
--  based on project from here:
--  https://www.youtube.com/watch?v=ccYLb7cLB1I

metaballs = {}

-- surface dimensions
-- this is the size of the image provided by metaballs.surface()
-- making this too high will severly drop frame rate
metaballs.w = 160
metaballs.h = 120

metaballs.balls = {}
metaballs.canvas = love.graphics.newCanvas(metaballs.w, metaballs.h)

local shaderCode = [[
#line 30
	vec3 hsl2rgb(float h, float s, float l) {
	    vec3 rgb = clamp(abs(mod(h * 6.0 + vec3(0.0, 4.0, 2.0), 6.0) - 3.0) - 1.0, 0.0, 1.0);
	    return l + s * (rgb - 0.5) * (1.0 - abs(2.0 * l - 1.0));
	}

	extern vec3 balls[%NBALLS%];

	vec4 effect(vec4 color, Image texture, vec2 uv, vec2 fc) {
		float sum = 0;

		for(int i = 0; i < %NBALLS%; ++i) {
			float d  = distance(balls[i].xy, fc);
			sum += 100 * balls[i].z / d;
		}

		return vec4(hsl2rgb(sum / 255 / 255, .5, sum / 255 / 255), 1);
	}
]]

-- add metaballs
function metaballs.create(num)
	for i=1, num do
		table.insert(metaballs.balls, {
			x = love.math.random(0,metaballs.w),
			y = love.math.random(0,metaballs.h),
			vx = love.math.random(10,500)/10,
			vy = love.math.random(10,500)/10,
			r = love.math.random(500,6000)
		})
	end

	metaballs.shader = love.graphics.newShader(shaderCode:gsub('%%NBALLS%%', num), nil)
	local sballs = {}
	for i = 1, num do
		local b = metaballs.balls[i]
		for i = 1, num do sballs[i] = { b.x, b.y, b.r } end
	end
	metaballs.shader:send('balls', unpack(sballs))
	metaballs.shaderBalls = sballs
end

-- create a drawable image surface of the metaballs
function metaballs.surface()
	metaballs.canvas:renderTo(function()
		metaballs.shader:send('balls', unpack(metaballs.shaderBalls))
		local old = love.graphics.getShader()
		love.graphics.setShader(metaballs.shader)
		love.graphics.rectangle('fill', 0, 0, metaballs.w, metaballs.h)
		love.graphics.setShader(old)
	end)
	return metaballs.canvas
end


function metaballs.update(dt)
	if #metaballs.balls < 0 then return end

	for i,b in ipairs(metaballs.balls) do
		-- bounce on the edges of the screen
		b.x = math.max(0,math.min(metaballs.w, b.x + b.vx *dt))
		b.y = math.max(0,math.min(metaballs.h, b.y + b.vy *dt))
		metaballs.shaderBalls[i] = { b.x, b.y, metaballs.shaderBalls[i][3] }
		if (b.x >= metaballs.w or b.x <= 0) then b.vx = b.vx * -1 end
		if (b.y >= metaballs.h or b.y <= 0) then b.vy = b.vy * -1 end
	end
end
